@@ -1,7 +1,7 @@
 /*
 An implementation of indicator object showing menus from applications.
 
-Copyright 2010 Canonical Ltd.
+Copyright 2010-2013 Canonical Ltd.
 
 Authors:
     Ted Gould <ted@canonical.com>
@@ -23,6 +23,8 @@ with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "config.h"
 #endif
 
+#include <stdlib.h> /* exit() */
+
 #include <X11/Xlib.h>
 #include <gdk/gdkx.h>
 #include <gio/gio.h>
@@ -81,8 +83,6 @@ struct _IndicatorAppmenuClass {
 struct _IndicatorAppmenu {
 	IndicatorObject parent;
 
-	gulong retry_registration;
-
 	WindowMenu * default_app;
 	GHashTable * apps;
 
@@ -106,8 +106,6 @@ struct _IndicatorAppmenu {
 	GDBusConnection * bus;
 	guint owner_id;
 	guint dbus_registration;
-
-	GHashTable * destruction_timers;
 };
 
 
@@ -183,7 +181,6 @@ static void active_window_changed                                    (BamfMatche
                                                                       BamfView * newview,
                                                                       gpointer user_data);
 static GQuark error_quark                                            (void);
-static gboolean retry_registration                                   (gpointer user_data);
 static void bus_method_call                                          (GDBusConnection * connection,
                                                                       const gchar * sender,
                                                                       const gchar * object_path,
@@ -195,15 +192,9 @@ static void bus_method_call                                          (GDBusConne
 static void on_bus_acquired                                          (GDBusConnection * connection,
                                                                       const gchar * name,
                                                                       gpointer user_data);
-static void on_name_acquired                                         (GDBusConnection * connection,
-                                                                      const gchar * name,
-                                                                      gpointer user_data);
 static void on_name_lost                                             (GDBusConnection * connection,
                                                                       const gchar * name,
                                                                       gpointer user_data);
-static void menus_destroyed                                          (GObject * menus,
-                                                                      gpointer user_data);
-static void source_unregister                                        (gpointer user_data);
 static GVariant * unregister_window                                  (IndicatorAppmenu * iapp,
                                                                       guint windowid);
 
@@ -275,7 +266,6 @@ indicator_appmenu_init (IndicatorAppmenu *self)
 	self->active_window = NULL;
 	self->active_stubs = STUBS_UNKNOWN;
 	self->close_item = NULL;
-	self->retry_registration = 0;
 	self->bus = NULL;
 	self->owner_id = 0;
 	self->dbus_registration = 0;
@@ -287,9 +277,6 @@ indicator_appmenu_init (IndicatorAppmenu *self)
 	self->desktop_windows = g_hash_table_new(g_direct_hash, g_direct_equal);
 	self->desktop_menu = NULL; /* Starts NULL until found */
 
-	/* Set up the hashtable of destruction timers */
-	self->destruction_timers = g_hash_table_new_full(g_direct_hash, g_direct_equal, NULL, source_unregister);
-
 	build_window_menus(self);
 
 	/* Get the default BAMF matcher */
@@ -309,29 +296,14 @@ indicator_appmenu_init (IndicatorAppmenu *self)
 	find_desktop_windows(self);
 
 	/* Request a name so others can find us */
-	retry_registration(self);
-
-	return;
-}
-
-/* If we weren't able to register on the bus, then we need
-   to try it all again. */
-static gboolean
-retry_registration (gpointer user_data)
-{
-	g_return_val_if_fail(IS_INDICATOR_APPMENU(user_data), FALSE);
-	IndicatorAppmenu * iapp = INDICATOR_APPMENU(user_data);
-
-	iapp->owner_id = g_bus_own_name (G_BUS_TYPE_SESSION,
+	self->owner_id = g_bus_own_name (G_BUS_TYPE_SESSION,
 	                                 DBUS_NAME,
 	                                 G_BUS_NAME_OWNER_FLAGS_NONE,
-	                                 iapp->dbus_registration == 0 ? on_bus_acquired : NULL,
-	                                 on_name_acquired,
+	                                 on_bus_acquired,
+	                                 NULL,
 	                                 on_name_lost,
-	                                 g_object_ref(iapp),
-	                                 g_object_unref);
-
-	return TRUE;
+	                                 self,
+	                                 NULL);
 }
 
 static void
@@ -341,6 +313,11 @@ on_bus_acquired (GDBusConnection * connection, const gchar * name,
 	IndicatorAppmenu * iapp = INDICATOR_APPMENU(user_data);
 	GError * error = NULL;
 
+	if (connection == NULL) {
+		g_critical ("Unable to get session bus.");
+		exit (0);
+	}
+
 	iapp->bus = connection;
 
 	/* Now register our object on our new connection */
@@ -355,19 +332,7 @@ on_bus_acquired (GDBusConnection * connection, const gchar * name,
 	if (error != NULL) {
 		g_critical("Unable to register the object to DBus: %s", error->message);
 		g_error_free(error);
-		g_bus_unown_name(iapp->owner_id);
-		iapp->owner_id = 0;
-		iapp->retry_registration = g_timeout_add_seconds(1, retry_registration, iapp);
-		return;
 	}
-
-	return;	
-}
-
-static void
-on_name_acquired (GDBusConnection * connection, const gchar * name,
-                  gpointer user_data)
-{
 }
 
 static void
@@ -386,8 +351,6 @@ on_name_lost (GDBusConnection * connection, const gchar * name,
 	/* We can rest assured no one will register with us, but let's
 	   just ensure we're not showing anything. */
 	switch_default_app(iapp, NULL, NULL);
-
-	iapp->owner_id = 0;
 }
 
 /* Object refs decrement */
@@ -396,26 +359,12 @@ indicator_appmenu_dispose (GObject *object)
 {
 	IndicatorAppmenu * iapp = INDICATOR_APPMENU(object);
 
-	/* Don't register if we're dying! */
-	if (iapp->retry_registration != 0) {
-		g_source_remove(iapp->retry_registration);
-		iapp->retry_registration = 0;
-	}
-
 	if (iapp->dbus_registration != 0) {
 		g_dbus_connection_unregister_object(iapp->bus, iapp->dbus_registration);
 		/* Don't care if it fails, there's nothing we can do */
 		iapp->dbus_registration = 0;
 	}
 
-	if (iapp->destruction_timers != NULL) {
-		/* These are in dispose and not finalize becuase the dereference
-		   function removes timers that could need the object to be in
-		   a valid state, so it's better to have them in dispose */
-		g_hash_table_destroy(iapp->destruction_timers);
-		iapp->destruction_timers = NULL;
-	}
-
 	if (iapp->bus != NULL) {
 		g_object_unref(iapp->bus);
 		iapp->bus = NULL;
@@ -627,9 +576,6 @@ new_window (BamfMatcher * matcher, BamfView * view, gpointer user_data)
 	IndicatorAppmenu * iapp = INDICATOR_APPMENU(user_data);
 	guint32 xid = bamf_window_get_xid(window);
 
-	/* Make sure we don't destroy it later */
-	g_hash_table_remove(iapp->destruction_timers, GUINT_TO_POINTER(xid));
-
 	if (bamf_window_get_window_type(window) != BAMF_WINDOW_DESKTOP) {
 		return;
 	}
@@ -651,32 +597,6 @@ new_window (BamfMatcher * matcher, BamfView * view, gpointer user_data)
 	return;
 }
 
-typedef struct _destroy_data_t destroy_data_t;
-struct _destroy_data_t {
-	IndicatorAppmenu * iapp;
-	guint32 xid;
-};
-
-/* Timeout to finally cleanup the window.  Causes is to ignore glitches that
-   come from BAMF/WNCK. */
-static gboolean
-destroy_window_timeout (gpointer user_data)
-{
-	destroy_data_t * destroy_data = (destroy_data_t *)user_data;
-	g_hash_table_steal(destroy_data->iapp->destruction_timers, GUINT_TO_POINTER(destroy_data->xid));
-	unregister_window(destroy_data->iapp, destroy_data->xid);
-	return FALSE; /* free's data through source deregistration */
-}
-
-/* Unregisters the source in the hash table when it gets removed.  This ensure
-   we don't leave any timeouts around */
-static void
-source_unregister (gpointer user_data)
-{
-	g_source_remove(GPOINTER_TO_UINT(user_data));
-	return;
-}
-
 /* When windows leave us, this function gets called */
 static void
 old_window (BamfMatcher * matcher, BamfView * view, gpointer user_data)
@@ -689,12 +609,7 @@ old_window (BamfMatcher * matcher, BamfView * view, gpointer user_data)
 	BamfWindow * window = BAMF_WINDOW(view);
 	guint32 xid = bamf_window_get_xid(window);
 
-	destroy_data_t * destroy_data = g_new0(destroy_data_t, 1);
-	destroy_data->iapp = iapp;
-	destroy_data->xid = xid;
-
-	guint source_id = g_timeout_add_seconds_full(G_PRIORITY_LOW, 5, destroy_window_timeout, destroy_data, g_free);
-	g_hash_table_replace(iapp->destruction_timers, GUINT_TO_POINTER(xid), GUINT_TO_POINTER(source_id));
+	unregister_window(iapp, xid);
 
 	return;
 }
@@ -1137,18 +1052,15 @@ active_window_changed (BamfMatcher * matcher, BamfView * oldview, BamfView * new
 /* Respond to the menus being destroyed.  We need to deregister
    and make sure we weren't being shown.  */
 static void
-menus_destroyed (GObject * menus, gpointer user_data)
+menus_destroyed (IndicatorAppmenu * iapp, guint windowid)
 {
 	gboolean reload_menus = FALSE;
-	WindowMenu * wm = WINDOW_MENU(menus);
-	IndicatorAppmenu * iapp = INDICATOR_APPMENU(user_data);
+	WindowMenu * wm = g_hash_table_lookup(iapp->apps, GUINT_TO_POINTER(windowid));
+	g_return_if_fail (wm != NULL);
 
-	guint32 xid = window_menu_get_xid(wm);
-	g_return_if_fail(xid != 0);
+	g_hash_table_steal(iapp->apps, GUINT_TO_POINTER(windowid));
 
-	g_hash_table_steal(iapp->apps, GUINT_TO_POINTER(xid));
-
-	g_debug("Removing menus for %d", xid);
+	g_debug("Removing menus for %d", windowid);
 
 	if (iapp->desktop_menu == wm) {
 		iapp->desktop_menu = NULL;
@@ -1168,7 +1080,7 @@ menus_destroyed (GObject * menus, gpointer user_data)
 		switch_default_app(iapp, NULL, NULL);
 	}
 
-	return;
+	g_object_unref(wm);
 }
 
 /* A new window wishes to register it's windows with us */
@@ -1178,9 +1090,6 @@ register_window (IndicatorAppmenu * iapp, guint windowid, const gchar * objectpa
 {
 	g_debug("Registering window ID %d with path %s from %s", windowid, objectpath, sender);
 
-	/* Shouldn't do anything, but let's be sure */
-	g_hash_table_remove(iapp->destruction_timers, GUINT_TO_POINTER(windowid));
-
 	if (g_hash_table_lookup(iapp->apps, GUINT_TO_POINTER(windowid)) == NULL && windowid != 0) {
 		WindowMenu * wm = WINDOW_MENU(window_menu_dbusmenu_new(windowid, sender, objectpath));
 		g_return_val_if_fail(wm != NULL, FALSE);
@@ -1230,27 +1139,12 @@ unregister_window (IndicatorAppmenu * iapp, guint windowid)
 	g_return_val_if_fail(IS_INDICATOR_APPMENU(iapp), NULL);
 	g_return_val_if_fail(iapp->matcher != NULL, NULL);
 
-	/* Make sure we don't destroy it later */
-	g_hash_table_remove(iapp->destruction_timers, GUINT_TO_POINTER(windowid));
-
 	/* If it's a desktop window remove it from that table as well */
 	g_hash_table_remove(iapp->desktop_windows, GUINT_TO_POINTER(windowid));
 
 	emit_signal(iapp, "WindowUnregistered", g_variant_new ("(u)", windowid));
 
-	/* Now let's see if we've got a WM object for it then
-	   we need to mark it as destroyed and unreference to
-	   actually destroy it. */
-	gpointer wm = g_hash_table_lookup(iapp->apps, GUINT_TO_POINTER(windowid));
-	if (wm != NULL) {
-		GObject * wmo = G_OBJECT(wm);
-
-		/* Using destroyed so that if the menus are shown
-		   they'll be switch and the current window gets
-		   updated as well. */
-		menus_destroyed(wmo, iapp);
-		g_object_unref(wmo);
-	}
+	menus_destroyed(iapp, windowid);
 
 	return NULL;
 }
